@@ -30,6 +30,9 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
+#ifndef SIMULATION_DISK_IO_HPP
+#define SIMULATION_DISK_IO_HPP
+
 #include "libtorrent/disk_interface.hpp"
 #include "libtorrent/hasher.hpp"
 #include "libtorrent/file_storage.hpp"
@@ -46,7 +49,7 @@ lt::sha1_hash generate_hash1(lt::piece_index_t const p, lt::file_storage const&
 lt::sha1_hash generate_hash2(lt::piece_index_t p, lt::file_storage const& fs
 	, lt::span<lt::sha256_hash> const hashes);
 lt::sha256_hash generate_block_hash(lt::piece_index_t p, int offset);
-void generate_block(char* b, lt::peer_request const& r);
+void generate_block(char* b, lt::peer_request r, int pad_bytes, int piece_size);
 std::shared_ptr<lt::torrent_info> create_test_torrent(int piece_size
 	, int num_pieces, lt::create_flags_t flags, int num_files = 1);
 lt::add_torrent_params create_test_torrent(
@@ -123,3 +126,6 @@ struct test_disk
 	int space_left = std::numeric_limits<int>::max();
 };
 
+std::ostream& operator<<(std::ostream& os, existing_files_mode const mode);
+
+#endif
