@@ -204,9 +204,12 @@ namespace {
 
 		do
 		{
-			TORRENT_TRY {
+			try
+			{
 				buffer.resize(destlen);
-			} TORRENT_CATCH (std::exception const&) {
+			}
+			catch (std::exception const&)
+			{
 				ec = errors::no_memory;
 				return;
 			}
