@@ -1,6 +1,9 @@
 /*
 
-Copyright (c) 2013, Arvid Norberg
+Copyright (c) 2018, Steven Siloti
+Copyright (c) 2013-2020, Arvid Norberg
+Copyright (c) 2016, 2018-2019, Alden Torres
+Copyright (c) 2020, Paul-Louis Ageneau
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,20 +40,17 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/torrent_info.hpp"
 #include "libtorrent/hex.hpp" // for to_hex
 #include "libtorrent/time.hpp"
-#include "libtorrent/aux_/openssl.hpp"
+#include "libtorrent/ssl.hpp"
 
 #include "test.hpp"
 #include "test_utils.hpp"
 #include "setup_transfer.hpp"
 #include "settings.hpp"
 
+#if TORRENT_USE_SSL
+
 #include "libtorrent/aux_/disable_warnings_push.hpp"
-
 #include <boost/asio/connect.hpp>
-
-#ifdef TORRENT_USE_OPENSSL
-#include <boost/asio/ssl.hpp>
-
 #include "libtorrent/aux_/disable_warnings_pop.hpp"
 
 #include <functional>
@@ -58,6 +58,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <fstream>
 #include <iostream>
 
+#ifdef TORRENT_UTP_LOG_ENABLE
+#include "libtorrent/aux_/utp_stream.hpp"
+#endif
+
 using namespace std::placeholders;
 using namespace lt;
 using std::ignore;
@@ -102,11 +106,8 @@ bool on_alert(alert const* a)
 	{
 		++peer_disconnects;
 		string_view const cat = e->error.category().name();
-		if (cat == boost::asio::error::get_ssl_category().name()
-#if BOOST_VERSION >= 106400
-			|| cat == boost::asio::ssl::error::get_stream_category().name()
-#endif
-			)
+		if (cat == ssl::error::get_ssl_category().name()
+			|| cat == ssl::error::get_stream_category().name())
 			++ssl_peer_disconnects;
 
 		std::printf("--- peer_errors: %d ssl_disconnects: %d\n"
@@ -119,11 +120,8 @@ bool on_alert(alert const* a)
 		++peer_errors;
 
 		string_view const cat = e->error.category().name();
-		if (cat == boost::asio::error::get_ssl_category().name()
-#if BOOST_VERSION >= 106400
-			|| cat == boost::asio::ssl::error::get_stream_category().name()
-#endif
-			)
+		if (cat == ssl::error::get_ssl_category().name()
+			|| cat == ssl::error::get_stream_category().name())
 			++ssl_peer_disconnects;
 
 		std::printf("--- peer_errors: %d ssl_disconnects: %d\n"
@@ -140,9 +138,13 @@ void test_ssl(int const test_idx, bool const use_utp)
 	session_proxy p1;
 	session_proxy p2;
 
+#ifdef TORRENT_UTP_LOG_ENABLE
+	lt::aux::set_utp_stream_logging(use_utp);
+#endif
+
 	test_config_t const& test = test_config[test_idx];
 
-	std::printf("\n%s TEST: %s Protocol: %s\n\n", time_now_string()
+	std::printf("\n%s TEST: %s Protocol: %s\n\n", time_now_string().c_str()
 		, test.name, use_utp ? "uTP": "TCP");
 
 	// in case the previous run was terminated
@@ -168,7 +170,7 @@ void test_ssl(int const test_idx, bool const use_utp)
 	// if a peer fails once, don't try it again
 	sett.set_int(settings_pack::max_failcount, 1);
 
-	lt::session ses1(sett, {});
+	lt::session ses1(session_params{sett, {}});
 
 	// this +20 is here to use a different port as ses1
 	port += 20;
@@ -181,7 +183,7 @@ void test_ssl(int const test_idx, bool const use_utp)
 
 	sett.set_str(settings_pack::listen_interfaces, listen_iface);
 
-	lt::session ses2(sett, {});
+	lt::session ses2(session_params{sett, {}});
 
 	wait_for_listen(ses1, "ses1");
 	wait_for_listen(ses2, "ses2");
@@ -192,7 +194,7 @@ void test_ssl(int const test_idx, bool const use_utp)
 	create_directory("tmp1_ssl", ec);
 	std::ofstream file("tmp1_ssl/temporary");
 	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary"
-		, 16 * 1024, 13, false, combine_path("..", combine_path("ssl", "root_ca_cert.pem")));
+		, 16 * 1024, 13, false, {}, combine_path("..", combine_path("ssl", "root_ca_cert.pem")));
 	file.close();
 
 	add_torrent_params addp;
@@ -240,8 +242,8 @@ void test_ssl(int const test_idx, bool const use_utp)
 	// connect the peers after setting the certificates
 	if (test.use_ssl_ports == false) port += 20;
 	std::printf("\n\n%s: ses1: connecting peer port: %d\n\n\n"
-		, time_now_string(), port);
-	tor1.connect_peer(tcp::endpoint(address::from_string("127.0.0.1", ec)
+		, time_now_string().c_str(), port);
+	tor1.connect_peer(tcp::endpoint(make_address("127.0.0.1", ec)
 		, std::uint16_t(port)));
 
 	const int timeout = 40;
@@ -303,9 +305,9 @@ void test_ssl(int const test_idx, bool const use_utp)
 		TEST_EQUAL(peer_errors > 0, test.peer_errors > 0);
 	}
 
-	char const* now = time_now_string();
-	std::printf("%s: EXPECT: %s\n", now, test.expected_to_complete ? "SUCCEESS" : "FAILURE");
-	std::printf("%s: RESULT: %s\n", now, tor2.status().is_seeding ? "SUCCEESS" : "FAILURE");
+	std::string const now = time_now_string();
+	std::printf("%s: EXPECT: %s\n", now.c_str(), test.expected_to_complete ? "SUCCEESS" : "FAILURE");
+	std::printf("%s: RESULT: %s\n", now.c_str(), tor2.status().is_seeding ? "SUCCEESS" : "FAILURE");
 	TEST_EQUAL(tor2.status().is_seeding, test.expected_to_complete);
 
 	// this allows shutting down the sessions in parallel
@@ -313,13 +315,6 @@ void test_ssl(int const test_idx, bool const use_utp)
 	p2 = ses2.abort();
 }
 
-std::string password_callback(std::size_t /*length*/, boost::asio::ssl::context::password_purpose p
-	, std::string pw)
-{
-	if (p != boost::asio::ssl::context::for_reading) return "";
-	return pw;
-}
-
 struct attack_t
 {
 	// flags controlling the connection attempt
@@ -359,7 +354,7 @@ const int num_attacks = sizeof(attacks)/sizeof(attacks[0]);
 bool try_connect(lt::session& ses1, int port
 	, std::shared_ptr<torrent_info> const& t, std::uint32_t flags)
 {
-	using boost::asio::ssl::context;
+	using ssl::context;
 
 	std::printf("\nMALICIOUS PEER TEST: ");
 	if (flags & invalid_certificate) std::printf("invalid-certificate ");
@@ -376,16 +371,17 @@ bool try_connect(lt::session& ses1, int port
 	std::printf(" port: %d\n", port);
 
 	error_code ec;
-	boost::asio::io_service ios;
+	boost::asio::io_context ios;
 
 	// create the SSL context for this torrent. We need to
 	// inject the root certificate, and no other, to
 	// verify other peers against
-	context ctx(context::sslv23);
+	context ctx(context::tls);
 
 	ctx.set_options(context::default_workarounds
-		| boost::asio::ssl::context::no_sslv2
-		| boost::asio::ssl::context::single_dh_use);
+		| context::no_sslv2
+		| context::no_sslv3
+		| context::single_dh_use);
 
 	// we're a malicious peer, we don't have any interest
 	// in verifying peers
@@ -413,10 +409,12 @@ bool try_connect(lt::session& ses1, int port
 	if (flags & (valid_certificate | invalid_certificate))
 	{
 		std::printf("set_password_callback\n");
-		ctx.set_password_callback(std::bind(&password_callback, _1, _2, "test"), ec);
+		ctx.set_password_callback(
+			[](std::size_t, context::password_purpose) { return "test"; }
+			, ec);
 		if (ec)
 		{
-			std::printf("Failed to set certificate password callback: %s\n"
+			std::printf("Failed to set certificate passphrase: %s\n"
 				, ec.message().c_str());
 			TEST_CHECK(!ec);
 			return false;
@@ -450,11 +448,11 @@ bool try_connect(lt::session& ses1, int port
 		}
 	}
 
-	boost::asio::ssl::stream<tcp::socket> ssl_sock(ios, ctx);
+	ssl::stream<tcp::socket> ssl_sock(ios, ctx);
 
 	std::printf("connecting 127.0.0.1:%d\n", port);
 	ssl_sock.lowest_layer().connect(tcp::endpoint(
-		address_v4::from_string("127.0.0.1"), std::uint16_t(port)), ec);
+		make_address_v4("127.0.0.1"), std::uint16_t(port)), ec);
 	print_alerts(ses1, "ses1", true, true, &on_alert);
 
 	if (ec)
@@ -467,9 +465,10 @@ bool try_connect(lt::session& ses1, int port
 
 	if (flags & valid_sni_hash)
 	{
-		std::string name = aux::to_hex(t->info_hash());
+		std::string name = aux::to_hex(t->info_hashes().v1);
 		std::printf("SNI: %s\n", name.c_str());
-		aux::openssl_set_tlsext_hostname(ssl_sock.native_handle(), name.c_str());
+		ssl::set_host_name(ssl::get_handle(ssl_sock), name, ec);
+		TEST_CHECK(!ec);
 	}
 	else if (flags & invalid_sni_hash)
 	{
@@ -480,11 +479,12 @@ bool try_connect(lt::session& ses1, int port
 			name += hex_alphabet[rand() % 16];
 
 		std::printf("SNI: %s\n", name.c_str());
-		aux::openssl_set_tlsext_hostname(ssl_sock.native_handle(), name.c_str());
+		ssl::set_host_name(ssl::get_handle(ssl_sock), name, ec);
+		TEST_CHECK(!ec);
 	}
 
 	std::printf("SSL handshake\n");
-	ssl_sock.handshake(boost::asio::ssl::stream_base::client, ec);
+	ssl_sock.handshake(ssl::stream_base::client, ec);
 
 	print_alerts(ses1, "ses1", true, true, &on_alert);
 	if (ec)
@@ -502,7 +502,7 @@ bool try_connect(lt::session& ses1, int port
 	// fill in the info-hash
 	if (flags & valid_bittorrent_hash)
 	{
-		std::memcpy(handshake + 28, &t->info_hash()[0], 20);
+		std::memcpy(handshake + 28, &t->info_hashes().v1[0], 20);
 	}
 	else
 	{
@@ -541,7 +541,7 @@ bool try_connect(lt::session& ses1, int port
 		return false;
 	}
 
-	if (memcmp(buf + 28, &t->info_hash()[0], 20) != 0)
+	if (memcmp(buf + 28, t->info_hashes().v1.data(), 20) != 0)
 	{
 		std::printf("invalid info-hash in bittorrent handshake\n");
 		return false;
@@ -570,14 +570,14 @@ void test_malicious_peer()
 	sett.set_bool(settings_pack::enable_upnp, false);
 	sett.set_bool(settings_pack::enable_natpmp, false);
 
-	lt::session ses1(sett, {});
+	lt::session ses1(session_params{sett, {}});
 	wait_for_listen(ses1, "ses1");
 
 	// create torrent
 	create_directory("tmp3_ssl", ec);
 	std::ofstream file("tmp3_ssl/temporary");
 	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary"
-		, 16 * 1024, 13, false, combine_path("..", combine_path("ssl", "root_ca_cert.pem")));
+		, 16 * 1024, 13, false, {}, combine_path("..", combine_path("ssl", "root_ca_cert.pem")));
 	file.close();
 
 	TEST_CHECK(!t->ssl_cert().empty());
@@ -639,5 +639,5 @@ TORRENT_TEST(tcp_config7) { test_ssl(7, false); }
 TORRENT_TEST(tcp_config8) { test_ssl(8, false); }
 #else
 TORRENT_TEST(disabled) {}
-#endif // TORRENT_USE_OPENSSL
+#endif // TORRENT_SSL_PEERS
 
